from typing import Dict
from django.db.models import QuerySet

def dict_values(target:Dict,*fields):
    '''
    Retrieve a list of items specified by `*fields` on the `target` dictionary.
    The list is packed in the same order as the fields are provided
    '''
    lst=[]
    for field in fields:
        lst.append(target[field])
    return lst


def value_list(queryset:QuerySet, field: str):
    '''
    Generate an actual value list of one item per entry.
    This is meant to make it easier to retrieve list of items
    from `queryset` and we want an item
    '''
    return list(map(lambda p: p[0], list(queryset.values_list(field))))


from typing import Any, Dict
from django import template
from django.urls.resolvers import ResolverMatch

register = template.Library()


@register.simple_tag(name="active_path", takes_context=True)
def active_path(
    context: Dict[str, Any],
    path_name: str,
    output_when_true="active",
    output_when_false="",
):
    match: ResolverMatch = context.request.resolver_match
    is_matched = False
    if ":" in path_name:
        data = path_name.split(":")
        is_matched = match.app_name == data[0] and match.url_name == data[1]
    else:
        is_matched = path_name == match.url_name

    return output_when_true if is_matched else output_when_false


@register.simple_tag(name="active_path_in", takes_context=True)
def active_path_in(
    context: Dict[str, Any],
    *path_names: str,
    output_when_true="active",
    output_when_false="",
):
    match: ResolverMatch = context.request.resolver_match
    is_matched = False
    
    for path in path_names:
        if ":" in path:
            data = path.split(":")
            is_matched = match.app_name == data[0] and match.url_name == data[1]
        else:
            is_matched = path == match.url_name
        if is_matched: break

    return output_when_true if is_matched else output_when_false


@register.simple_tag(name="active_app", takes_context=True)
def active_app(
    context: Dict[str, Any],
    app_name: str,
    output_when_true="active",
    output_when_false="",
):
    return (
        output_when_true
        if context.request.resolver_match.app_name == app_name
        else output_when_false
    )






def generate_day_interval_choices(divisions_per_hour=4):
    '''
    Generates a tuple of fixed time intervals in a day.
    Each item consists of two `str` items: `(time, time)`.
    `divisions_per_hour` determines the number of divisions
    in each hour.
    '''
    HOURS_IN_DAY = 24

    total_divisions = divisions_per_hour * HOURS_IN_DAY
    interval=60//divisions_per_hour
    result:list[tuple[str,str]] = []
    track_hour = 0
    track_minute = 0

    for _ in range(0, total_divisions):
        hour_str = f"0{track_hour}" if track_hour < 10 else f"{track_hour}"
        minute_str = f"0{track_minute}" if track_minute < 10 else f"{track_minute}"
        time = f"{hour_str}:{minute_str}"
        choice=time, time
        result.append(choice)

        track_minute += interval

        if track_minute >= 60:
            track_minute = 0
            track_hour += 1

    return tuple(result)
